<?php
require_once ('op5/license/op5lic.php');
require_once ('op5/sysinfo.php');
class ninja_license {
	const THRESHOLD = 0.95;

	/**
	 * Holds the instance of the op5lic class for this session
	 *
	 * @var op5lic
	 */
	private $lic = false;
	/**
	 * Interface to get usage statistics
	 *
	 * @var op5sysinfo
	 */
	private $sysinfo = false;

	/**
	 * Create hooks and initialize libraries
	 */
	public function __construct() {
		Event::add('system.post_controller_constructor',
			array ($this,'check_license'));
		Event::add('ninja.show_login', array ($this,'show_login'));
		Event::add('system.pre_controller', array ($this,'cleanup_session'));

		$this->sysinfo = new op5sysinfo();
	}

	/**
	 * Hook to add some global notifications if license needs to
	 */
	public function check_license() {
		$controller = Event::$data;

		if ($controller instanceof Ninja_Controller) {
			if ($this->lic === false) {
				$this->lic = new op5lic();
			}

			foreach ($this->lic->fetch_load_messages() as $msg) {
				$controller->add_global_notification($msg);
			}

			try {
				$monitor_usage = $this->sysinfo->get_monitor_usage();
				if (!$this->lic->query_threshold('monitor', $monitor_usage,
					self::THRESHOLD, $monitor_messages)) {
					foreach ($monitor_messages as $msg) {
						$controller->add_global_notification($msg);
					}
				}
			} catch (op5LivestatusException $e) {
				/*
				 * Don't bubble this exception further. Just add a notice bar.
				 * Bubble this further means exposing the license library code
				 */
				$controller->add_global_notification($e->getMessage());
			}
		}
	}

	/**
	 * Hook to replace login screen if system is unlicensed
	 */
	public function show_login() {
		$controller = Event::$data;
		if ($this->lic === false) {
			$this->lic = new op5lic();
		}
		if ($this->lic->fetch_property('unlicensed')) {
			$controller->template = $controller->add_view('license_call_me');
		}
	}

	/**
	 * Destroy session if unlicensed.
	 * That means, we shouldn't be able to be logged in at all to ninja
	 */
	public function cleanup_session() {
		/*
		 * We should destroy the session and disable auth subsystem here if
		 * property unlicensed is set, but we can't until op5lic stops trigger a
		 * ninja request, which trigger this hook again...
		 */
		if (false) {
			Session::destroy();

			Auth::disable_auth_subsystem();
		}
	}
}

new ninja_license();